<?php
/**
 * Convert MOD files to NinjaTracker+ format.
 * 
 * by Jesse Blue / Ninjaforce
 *
 * 
 * Jul 29, 2018 - Initial release
 * Nov 25, 2019 - Added the option to save as zip
 *              - Added the option to save as either .ntp or #D50008
 * Jan 02, 2022 - Added streaming option
 * Jan 16, 2022 - Added option to output conversion information or include in zip file
 */

$output = '';
$zip = 1;
$resulttype = 0;
$streamtype = 1;
$showinformation = 0;

if(isset($_POST['resulttype']) && in_array($_POST['resulttype'], [0, 1]))
{
    $resulttype = $_POST['resulttype'];
}

if(isset($_POST['streamtype']) && in_array($_POST['streamtype'], [0, 1, 2]))
{
    $streamtype = (int) $_POST['streamtype'];
}

if(isset($_POST['resulttype']) && !isset($_POST['zip']))
{
    $zip = 0;
}

if(isset($_POST['showinformation']))
{
    $showinformation = 1;
}

if(!isset($_FILES['source_mod']) || empty($_FILES['source_mod']))
{
    return;
}

try {
    include(__DIR__ . DIRECTORY_SEPARATOR . 'ntpconverter_lib.php');

    if (
        !isset($_FILES['source_mod']['error']) ||
        is_array($_FILES['source_mod']['error'])
    ) {
        throw new Exception('Unable to read uploaded file.');
    }

    switch ($_FILES['source_mod']['error']) {
        case UPLOAD_ERR_OK:
            break;
        case UPLOAD_ERR_NO_FILE:
            throw new Exception('No file sent.');
        case UPLOAD_ERR_INI_SIZE:
        case UPLOAD_ERR_FORM_SIZE:
            throw new Exception('Exceeded filesize limit.');
        default:
            throw new Exception('Unknown errors.');
    }

    if ($_FILES['source_mod']['size'] > 4000000) {
        throw new Exception('Exceeded filesize limit.');
    }

    $path_source_mod = $_FILES['source_mod']['tmp_name'];
    $path_output_ntp = tempnam(sys_get_temp_dir(), 'ntp');

    $ntpconverter = new ntpconverter($path_source_mod, $path_output_ntp, $streamtype);
    $result = $ntpconverter->convert();

    if($result['success'] === false)
    {
        throw new Exception($result['message']);
    }

    $name = trim($_FILES['source_mod']['name']);
    if(strlen($name) === 0)
    {
            $name = 'output';
    }

    if(strtoupper(substr($name,-4)) === '.MOD')
    {
            $name = substr($name, 0, -4);
    }

    $suffix = $resulttype == 0 ? '.ntp' : '#D50008';

    if($showinformation === 0)
    {
        if($zip == 1)
        {
            $path_output_ntpinfo = tempnam(sys_get_temp_dir(), 'ntpinfo');
            file_put_contents($path_output_ntpinfo, $result['message']);

            $path_zip_archive = tempnam(sys_get_temp_dir(), 'zip');
            $zip = new ZipArchive();
            $zip->open($path_zip_archive, ZIPARCHIVE::CREATE);
            $zip->addFile($path_output_ntp, $name.$suffix);
            $zip->addFile($path_output_ntpinfo, $name.'.txt');
            $zip->close();
        
            header("Content-type: application/zip");
            header("Content-Disposition: attachment; filename=".$name.".zip");
            header("Content-length: " . filesize($path_zip_archive));
            header("Pragma: no-cache");
            header("Expires: 0");
            readfile($path_zip_archive);
            @unlink($path_zip_archive);
            @unlink($path_output_ntpinfo);
        }
        else
        {
            header("Content-type: application/octet-stream");
            header("Content-Disposition: attachment; filename=".$name.$suffix);
            header("Content-length: " . filesize($path_output_ntp));
            header("Pragma: no-cache");
            header("Expires: 0");
            readfile($path_output_ntp);
        }
        @unlink($path_source_mod);
        @unlink($path_output_ntp);
        exit;
    }
    else
    {
        $output = $result['message'];
    }
}
catch(Exception $ex)
{
    $output = 'UNABLE TO CONVERT '.$_FILES['source_mod']['name'].':'.PHP_EOL.PHP_EOL.$ex->getMessage();
}
