<?php

require_once(__DIR__ . DIRECTORY_SEPARATOR . 'ntpconverter_lib.php');
  
/* the command line class to run the converter */

class run_converter
{
    private static function help()
    {
      echo "ntpconverter by ninjaforce" . PHP_EOL . PHP_EOL;
      echo "  help: `php ntpconverter.php mysong.MOD [stream_config]`" . PHP_EOL . PHP_EOL;
      echo "  stream_config is optional and is one of:" . PHP_EOL;
      echo "    [STREAM_FORBIDDEN, STREAM_ALLOW, STREAM_ENFORCE]" . PHP_EOL . PHP_EOL;
      exit();
    }

    public function parse_commandline()
    {
        if(isset($_SERVER['argv'][1]))
        {
            $mod_arg = $_SERVER['argv'][1];
            switch($mod_arg) {
              case '-h':
              case '--help':
                self::help();
            }

            if(!file_exists($mod_arg))
            {
                throw new Exception(sprintf('Unable to find MOD file %s', $mod_arg));
            }
        }
        else
        {
            throw new Exception('Please specify a MOD file!');
        }

        if(strtoupper(substr($mod_arg,-4)) === '.MOD')
        {
            $name = substr($mod_arg, 0, -4);
        }
        else
        {
            $name = $mod_arg;
        }

        $this->path_source_mod = $mod_arg;
        $this->path_output_ntp = $name . '.ntp';
        $this->stream_config = ntpconverter::CONFIG_STREAM_ALLOW;

        if(isset($_SERVER['argv'][2])) {
            $stream_arg = $_SERVER['argv'][2];
            switch(strtoupper($stream_arg)) {
              case 'STREAM_FORBIDDEN':
                $this->stream_config = ntpconverter::CONFIG_STREAM_FORBIDDEN;
                break;
              case 'STREAM_ALLOW':
                $this->stream_config = ntpconverter::CONFIG_STREAM_ALLOW;
                break;
              case 'STREAM_ENFORCE':
                $this->stream_config = ntpconverter::CONFIG_STREAM_ENFORCE;
                break;
            }
        } 

    }

    public function run()
    {
        $return_code = 0;

        try {
            $this->parse_commandline();

            $ntconverter = new ntpconverter($this->path_source_mod, $this->path_output_ntp, $this->stream_config);
            $result = $ntconverter->convert();
    
            if($result['success'] === false)
            {
                throw new Exception($result['message']);
            }
            else
            {
                echo $result['message'];
            }
        }
        catch(Exception $ex)
        {
            echo 'UNABLE TO CONVERT:' . PHP_EOL . $ex->getMessage(). PHP_EOL;
            $return_code = 255;
        }
        
        return $return_code;
    }
}

$ntconverter = new run_converter();
exit($ntconverter->run());
